Setup
=====

Include the main CodeMirror JavaScript in the HTML source:

    <script src="<path_to>/codemirror.js"></script>

Add the class `qfq-codemirror` to a `<textarea>` element. This class marks the element to be processed by `QfqNS
.Helper.codemirror()`.

Configuration
=============

Set the `data-config` attribute on the `<textarea>` element. The value has to be valid JSON. It will then be
deserialized and passed to CodeMirror during initialization. E.g.


    <textarea class="qfq-codemirror" data-config='{ "mode": "text/x-sql", "lineNumbers": true }'></textarea>

Refer to the [CodeMirror manual](https://codemirror.net/doc/manual.html) under section "Configuration" for possible
configuration options.

Mode
====

The configuration option `mode` sets the mode to be used by CodeMirror. To work properly, the corresponding
JavaScript file has to be loaded. For instance, `mode: "text/x-sql"` requires to include `code-mirror-mode/sql/sql.js`

    <script src="<path_to>/code-mirror-mode/sql/sql.js"></script>

Currently, only QFQ mode is supported. Support for additional modes can be enabled by adjusting `terser.js`, `package.json` or `copy.js`.

QFQ Mode
====

QFQ Mode was programmed by a member of the qfq development team
(Elias Villiger) as a CodeMirror 5 syntax highlighting mode.

Its main functionality is in the `javascript/src/Helper/codemirror/qfq.js` file and includes syntax
parsing and highlighting, hinting, and customizing a few options for qfq syntax.
In addition, a number of preexisting codemirror extensions (addons)
are delivered with qfq, such as matchbrackets, match-highlighting, etc.

The syntax highlighting works for Frontend Report Editor,
FormElement with `editorType=codemirror`, as well as the BackEnd t3editor.

## Functionality Overview

### QFQ Syntax highlighting

This is a parser for the qfq language. It finds the relevant qfq
keywords and concludes the context off of that (either SQL or
HTML). Depending on the context it tries to do some additional
highlighting in that language. It does not, however,
fully parse any SQL or HTML, so it may be inaccurate in those
subsections. The reason this wasn't implemented in mixed mode
was to still allow highlighting (and hinting) for qfq `{{variables}}`.

Please see the codemirror 5 manual and the code for how the
parser proceeds through the code.

### QFQ Hinting

The basics for QFQ hinting (e.g. table and column aliases) are
built as the document is parsed.  Not parsing the entire document
(setting a low `viewportMargin`) may produce inaccurate results.
Integration of the hinter is not perfect, but should be accurate
in most cases. The crucial function for connecting the parser
with both the hinter and the SQL capitalizer is `cm.getTokenAt`.

In the FrontEnd Report Editor, the current database schema is
automatically passed as a `tableColumns` config option. When a
database schema is present, it is evaluated and should only display
valid `table.column` combinations.

When no database schema is present (such as in the backend),
column and table names are recognized/guessed by considering some of the
surrounding syntax (`AS` keywords, table`.`column notation), but will
be incomplete.

QFQ variable suggestions are also generated by checking all the
appropriate `AS` column notations.

### Capitalize SQL

An auto capitalize feature is implemented for SQL keywords.
Whenever the space bar or an opening parenthesis is entered,
the preceding word is checked if it qualifies. The capitalizer
depends directly on the qfq parsing mode, which should provide a
good degree of accuracy.

To undo capitalization, use `Ctrl+Z`
or avoid hitting the spacebar after entering the word.

### Other features

Some other details are configured in the `javascript/src/Helper/codemirror/qfq.js` file, such as key
bindings for indentation (see End User Documentation).
Some of the functions in `qfq.js` are shared by various of the features,
which is why they are all packaged in one file.
To add/edit keywords, simply edit the corresponding keywords variable at the
beginning of the file.

## File Structure
In order to limit the number of files to be included the
following file structure is used:

### Javascript

JS Sources:
* `javascript/src/Helper/`
  * `codemirror/qfq.js` -> qfq mode syntax highlighting and hinting functionality, for FE and BE
  * `codemirror.js` -> QFQ integration of codemirror. This file is packaged separately (into qfq.min.js)
* `node_modules/codemirror/`
  * `lib/codemirror.js` -> Codemirror's own main code
  * `addon/search/match-highlighter.js`
  * `addon/hint/show-hint.js`
  * `addon/edit/matchbrackets.js`
  * `addon/edit/closebrackets.js`
  * `addon/selection/active-line.js`

JS Targets:
  * `Resources/Public/Javascript/`
    * `qfq.min.js`: `Helper/codemirror.js` has key QFQ functionality and is packaged into this file (among many others); without it the FE Report Editor doesn't work properly
    * `codemirror/codemirror-qfq.min.js`: All files needed to get qfq syntax highlighting mode with all its addons to work.
    * `codemirror/qfq.min.js`: `Helper/codemirror/qfq.js`, packaged separately (twice), so it can also be used for the BE t3editor. Needs to be called qfq.js in order to work with RequireJS.

### CSS
CSS Sources:
* `node_modules/codemirror/`
  * `lib/codemirror.css`
  * `addon/hint/show-hint.css`
  * `less/codemirror-custom.css`: contains any overrides / custom css, including qfqdark mode. This file is also included by the BE t3editor.

CSS Target:
* `Resources/Public/Css/codemirror.css`: includes all the above files
